@@ -113,6 +113,8 @@ sub main::generate($$) {
    print '    funcop_->prepareToShutdown();', "\n";
    print '}', "\n";
    print "\n";
+   my $nonMatchOutput = $model->getNumberOfOutputPorts() == 2;
+   print "\n";
    print 'void MY_OPERATOR_SCOPE::MY_OPERATOR::process(Tuple const & tuple, uint32_t port)', "\n";
    print '{', "\n";
    print '    bool passed = false;', "\n";
@@ -179,14 +181,15 @@ sub main::generate($$) {
    print "\n";
    print '  PyObject *value = 0;', "\n";
    print '  {', "\n";
-   print '  SplpyGIL lockdict;', "\n";
-   print '  PyObject * pyDict = PyDict_New();', "\n";
+   print '    SplpyGIL lockdict;', "\n";
+   print '    PyObject * pyDict = PyDict_New();', "\n";
+   print '    // convertAndAddToPythonDictionaryObject', "\n";
         for (my $i = 0; $i < $inputAttrs2Py; ++$i) {
             my $la = $iport->getAttributeAt($i);
-            print convertAndAddToPythonDictionaryObject($iport->getCppTupleName(), $i, $la->getSPLType(), $la->getName(), 'pyInNames_');
+            print convertAndAddToPythonDictionaryObject($iport->getCppTupleName(), $i, $la->getSPLType(), $la->getName(), 'pyInNames_', 'pyDict', "    ", $model->getContext()->getOutputDirectory());
         }
    print "\n";
-   print '  value = pyDict;', "\n";
+   print '    value = pyDict;', "\n";
    print '  }', "\n";
     } elsif ($pystyle eq 'tuple' || $pystyle_nt) { 
    print "\n";
@@ -201,23 +204,23 @@ sub main::generate($$) {
    print "\n";
    print '  PyObject *value = 0;', "\n";
    print '  {', "\n";
-   print '  SplpyGIL locktuple;', "\n";
-   print '  PyObject * pyTuple = PyTuple_New(';
+   print '    SplpyGIL locktuple;', "\n";
+   print '    PyObject * pyTuple = PyTuple_New(';
    print $inputAttrs2Py;
    print ');', "\n";
+   print '    // convertAndAddToPythonTupleObject', "\n";
         for (my $i = 0; $i < $inputAttrs2Py; ++$i) {
             my $la = $iport->getAttributeAt($i);
-            print convertAndAddToPythonTupleObject($iport->getCppTupleName(), $i, $la->getSPLType(), $la->getName());
+            print convertAndAddToPythonTupleObject($iport->getCppTupleName(), $i, $la->getSPLType(), $la->getName(), "    ", $model->getContext()->getOutputDirectory());
         }
    print "\n";
     if ($pystyle_nt) { 
    print "\n";
    print '    pyTuple = streamsx::topology::SplpyGeneral::pyCallObject(pyNamedtupleCls_, pyTuple);', "\n";
-   print "\n";
     } 
    print "\n";
    print "\n";
-   print '  value = pyTuple;', "\n";
+   print '    value = pyTuple;', "\n";
    print '  }', "\n";
     } 
    print "\n";
@@ -239,6 +242,14 @@ sub main::generate($$) {
    print '    }', "\n";
    print '    if (passed)', "\n";
    print '         submit(tuple, 0);', "\n";
+   print '    ';
+   if ($nonMatchOutput) {
+   print "\n";
+   print '    else', "\n";
+   print '         submit(tuple, 1);', "\n";
+   print '    ';
+   }
+   print "\n";
    print '}', "\n";
    print "\n";
    print 'void MY_OPERATOR_SCOPE::MY_OPERATOR::process(Punctuation const & punct, uint32_t port)', "\n";
